/*
 * Copyright (c) 2020 Jetro AS.
 *
 * SPDX-License-Identifier:
 */

#include <zephyr.h>
#include <string.h>
#include <stdio.h>

#include <device.h>
#include <drivers/gpio.h>

#include <modem/at_cmd.h>
#include <modem/at_params.h>
#include <modem/at_cmd_parser.h>

#define NR_OF_MEASURES		100
#define MS_BETWEEN_MEASURES	100

static struct rf_data_t
{
	int snr;
	int antenna_power;
} rf_data[NR_OF_MEASURES];

static K_SEM_DEFINE(button_sem, 0, 1);

static const struct device *gpio_led_dev;
static const uint8_t pin_led_nr = DT_GPIO_PIN(DT_ALIAS(led0), gpios);

static struct gpio_callback gpio_cb;

static void button_gpio_cb(const struct device *dev, struct gpio_callback *cb,
			   uint32_t pins)
{
	ARG_UNUSED(dev);
	ARG_UNUSED(cb);
	ARG_UNUSED(pins);

	/* Give semaphore. */
	k_sem_give(&button_sem);
}

static int configure_button()
{
	const struct device *gpio_dev;
	const char * const port = DT_GPIO_LABEL(DT_ALIAS(sw0), gpios);
	const uint8_t pin_nr = DT_GPIO_PIN(DT_ALIAS(sw0), gpios);

	gpio_dev = device_get_binding(port);
	if (!gpio_dev) {
		printk("Cannot get GPIO device\n");
		return -EIO;
	}

	/* Configure GPIO for interrupts. */
	gpio_pin_configure(gpio_dev, pin_nr, GPIO_INPUT | GPIO_PULL_UP);
	gpio_init_callback(&gpio_cb, button_gpio_cb, BIT(pin_nr));
	gpio_add_callback(gpio_dev, &gpio_cb);
	gpio_pin_interrupt_configure(gpio_dev, pin_nr, GPIO_INT_LEVEL_LOW);

	return 0;
}

static int configure_led()
{
	const char * const port = DT_GPIO_LABEL(DT_ALIAS(led0), gpios);

	gpio_led_dev = device_get_binding(port);
	if (!gpio_led_dev) {
		printk("Cannot get GPIO device\n");
		return -EIO;
	}

	/* Configure GPIO for interrupts. */
	gpio_pin_configure(gpio_led_dev, pin_led_nr, GPIO_OUTPUT_INACTIVE | DT_GPIO_FLAGS(DT_ALIAS(led0), gpios));

	return 0;
}

static int set_led(bool active)
{
	gpio_pin_set(gpio_led_dev, pin_led_nr, active ? 1 : 0);

	return 0;
}

void main(void)
{
	int err;
	char at_response[64];
	enum at_cmd_state at_state;
	struct at_param_list at_param_list;
	int snr;
	int antenna_power;

	err = configure_button();
	if (err) {
		printk("Failed to configure button: %d\n", err);
		return;
	}

	err = configure_led();
	if (err) {
		printk("Failed to configure led: %d\n", err);
		return;
	}

	err = at_params_list_init(&at_param_list, 3);
	if (err) {
		printk("Could not init AT params list, error: %d\n", err);
		return;
	}

#if CONFIG_BOARD_THINGY91_NRF9160NS
	/* Enable external antenna. */
	err = at_cmd_write("AT%XCOEX0", NULL, 0, NULL);
	if (err) {
		printk("Could not configure COEX0, error: %d\n", err);
		return;
	}
#endif /* CONFIG_BOARD_THINGY91_NRF9160NS */

	/* It's alive ...*/
	set_led(1);
	k_sleep(K_MSEC(100));
	set_led(0);

	/* Wait for start of test. */
	k_sleep(K_SECONDS(30));

	/* Take measurements. */
	for (int i=0; i<NR_OF_MEASURES; i++) {
		/* Start SNR measurement. */
		set_led(1);
		err = at_cmd_write("AT%XRFTEST=2,1,-90", at_response, 64, &at_state);
		if (err != 0) {
			if (err < 0) {
				printk("AT command failed: %d\n", err);
			} else {
				printk("AT command failed: %d, state: %d\n", err, at_state);
			}
			goto next_measurement;
		}
		set_led(0);

		/* Parse result. */
		printk("AT response: %s", at_response);

		err = at_parser_params_from_str(at_response, NULL, &at_param_list);
		if (err != 0) {
			printk("Failed parsing response: %d\n", err);
			goto next_measurement;
		}

		/* Parse SNR */
		err = at_params_int_get(&at_param_list, 1, &snr);
		if (err != 0) {
			printk("Failed parsing snr: %d\n", err);
			goto next_measurement;
		}
		rf_data[i].snr = snr;

		/* Parse antenna power */
		err = at_params_int_get(&at_param_list, 2, &antenna_power);
		if (err != 0) {
			printk("Failed antenna power: %d\n", err);
			goto next_measurement;
		}
		rf_data[i].antenna_power = antenna_power;

next_measurement:
		k_sleep(K_MSEC(MS_BETWEEN_MEASURES));
	}

	/* Wait for button press to print out measures. */
	k_sem_reset(&button_sem);
	printk("Measurements done waiting for keypress !\n");
	k_sem_take(&button_sem, K_FOREVER);

	for (int i=0; i<NR_OF_MEASURES; i++) {
		printk("Measurement: %d, snr: %d, antenna_power: %d\n", i, rf_data[i].snr, rf_data[i].antenna_power);
		k_sleep(K_MSEC(10));
	}

}
